--[[
    Torta: A World of Warcraft addon that helps you keep track of things
    Copyright (C) 2008-2009 Lawrence Patrick Calulo (Quezacolt)

    Torta is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Torta is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Torta.  If not, see <http://www.gnu.org/licenses/>.
]]
-- ====================================================================
--  CL3.lua
--  Contains code for performing operations on list data
-- ====================================================================
local Torta = Torta;
local Checklist3 = Torta.cl3;
local gui = Torta.gui;
local widgets = Torta.gui.elements;
local db;

local displayWindowPoints = { widgets.clDisplayWindow:GetPoint(1) };
local dispWnd = widgets.clDisplayWindow;
local tEditor = widgets.taskEditor;
local aEditor = widgets.alertsEditor;

local newListCount = 0;

-- We put search matches in here
local searchMatches = {};
-- Indices of the search matches, so we can pass the correct index to LoadList
local searchIndices = {};

Torta:RegisterRevision("$Revision: 26 $");

function Checklist3:InitializeChecklist()
  -- The db only becomes available after VARIABLES_LOADED, which is when this
  -- method is called.
  db = Torta.db;
  if type(db.profile.cfg.loadedList) == "number" then
    self:LoadList(db.profile.cfg.loadedList);
  else
    self:LoadList(nil)
  end
  
  self:ListLoadedCheck();
  
  if not self:IsEditTaskDialogOpen() then
    widgets.removeTaskButton:Disable();
  end
end

--[[
LoadList() takes a list ID (the list's index in db.lists)
When given false or nil as a parameter, the currently loaded list
is unloaded.
When given the ID of the currently loaded list, the currently
loaded list is unloaded.
If the Explorer is in search mode, this method adjusts the
given list ID to point to the correct list.
--]]
function Checklist3:LoadList(listID)
  local lists = db.profile.lists;
  self:HideTaskEditor();
  if self.isSearching then
    listID = searchIndices[listID];
  end
  if (not listID) or (Torta.gui.listData == lists[listID]) then
    -- "Unload" the list, show the no list loaded text, hide stuff
    gui.listData = nil;
    self.listData = nil;
    
    widgets.cl3HeaderText:SetText("Torta Checklist");
    db.profile.cfg.loadedList = false;
    self.openTask = nil;
  else
    -- Do the opposite - "load" the list!
    gui.listData = lists[listID];
    self.listData = lists[listID];
    FauxScrollFrame_SetOffset(Torta.gui.elements.clScrollBar, 0);
    getglobal(widgets.clScrollBar:GetName() .. "ScrollBar"):SetValue(0);
    widgets.cl3HeaderText:SetText(lists[listID].meta.listName);
    db.profile.cfg.loadedList = listID;
    self.openTask = nil;
  end
  gui:clUpdateScrollFrame();
  gui:exUpdateScrollFrame();
  self:ListLoadedCheck()
end


function Checklist3:GetLoadedList()
  return db.profile.lists[db.profile.cfg.loadedList];
end


function Checklist3:GetList(listID)
  return db.profile.lists[listID];
end


function Checklist3:ListLoadedCheck()
  if not self:GetLoadedList() then
    widgets.checkAllButton:Disable();
    widgets.uncheckAllButton:Disable();
    widgets.addTaskButton:Disable();
  else
    widgets.checkAllButton:Enable();
    widgets.uncheckAllButton:Enable();
    widgets.addTaskButton:Enable();
  end
end


function Checklist3:EditTask(taskID)
  if self.openTask and self.openTask == taskID then
    -- Close the task editor
    self.openTask = nil;
    self:HideTaskEditor();
  else
    -- Open the task editor
    self:ShowTaskEditor(taskID);
    widgets.editTaskEB:SetFocus();
    self.openTask = taskID;
    widgets.editTaskEB:SetText(self:GetLoadedList().text[taskID]);
  end
  gui:clUpdateScrollFrame();
end


function Checklist3:IsEditTaskDialogOpen()
  return not not self.openTask;
end


function Checklist3:ModifyOpenTask(newText)
  assert(type(self.openTask) == "number");
  local profile = db.profile;
  self:GetLoadedList().text[self.openTask] = newText;
end


function Checklist3:HideTaskEditor()
  dispWnd:ClearAllPoints();
  dispWnd:SetPoint(unpack(displayWindowPoints));
  dispWnd:SetPoint("BOTTOMRIGHT", widgets.clTaskBar, "TOPRIGHT");
  tEditor:Hide();
  self.openTask = nil;
  widgets.removeTaskButton:Disable();
  gui:clUpdateScrollFrame();
end


function Checklist3:ShowTaskEditor(taskID)
  -- "Internal" function. Use cl3:EditTask() to open the dialog for editing
  -- This only handles the GUI part.
  tEditor:Show();
  dispWnd:ClearAllPoints();
  dispWnd:SetPoint(unpack(displayWindowPoints));
  dispWnd:SetPoint("BOTTOMRIGHT", tEditor, "TOPRIGHT");
  getglobal(tEditor:GetName() .. "HeaderText"):SetText("Task Editor: " .. (taskID or "|cffff0000No task ID|r"))
  widgets.removeTaskButton:Enable();
  gui:clCheckButtons();
  gui:MakeTaskVisible(taskID);
  gui:clUpdateScrollFrame();
end


function Checklist3:UncheckAllTasks()
  local list = self:GetLoadedList();
  for i = 1, #list.stat do
    list.stat[i] = false;
  end
  gui:clUpdateScrollFrame();
end

function Checklist3:CheckAllTasks()
  local list = self:GetLoadedList();
  for i = 1, #list.stat do
    list.stat[i] = true;
  end
  gui:clUpdateScrollFrame();
end

-- The checkbox click handler
function Checklist3:OnCheckClicked(id)
  local list = self:GetLoadedList();
  if list then
    list.stat[id] = not list.stat[id];
  end
  gui:clUpdateScrollFrame();
end


function Checklist3:AddTask(insertIndex)
  local list = self:GetLoadedList();
  if not list then return end
  
  local index;
  if not insertIndex then
    index = (self.openTask or #list.text) + 1;
  else
    index = insertIndex;
  end
  tinsert(list.text, index, "");
  tinsert(list.stat, index, false);
  
  -- Check if we have an open task. If the opened task moved as a result
  -- of creating this new task, update the task edit dialog to make changes
  -- to the new location of the currently opened task.
  if self.openTask and insertIndex <= self.openTask then
    local correctOpenTaskID = self.openTask + 1;
    -- self:HideTaskEditor();
    self:EditTask(correctOpenTaskID);
  end
  
  gui:clUpdateScrollFrame();
end

function Checklist3:RemoveTask()
  local list = self:GetLoadedList();
  if not list then return end
  
  tremove(list.text, self.openTask);
  tremove(list.stat, self.openTask);
  gui:clUpdateScrollFrame();
  self:HideTaskEditor();
end

function Checklist3:DeleteTaskByID(id)
  local list = self:GetLoadedList();
  if not list then return end;
  
  tremove(list.text, id);
  tremove(list.stat, id);
  gui:clUpdateScrollFrame();
end


function Checklist3:CreateList()
  local name;
  for i = 1, 1000 do
    if not self:ListExists("New List " .. i) then
      name = "New List " .. i;
      break;
    end
  end
  tinsert(db.profile.lists, {
    meta = {
      listName = name
    },
    text = {},
    stat = {}
  });
  gui:exUpdateScrollFrame();
end

function Checklist3:DeleteList(index, confirmed)
  assert(index);
  if confirmed then
    local loadedListIndex = db.profile.cfg.loadedList;
    tremove(db.profile.lists, index);
    if loadedListIndex then
      if index < loadedListIndex then
        self:LoadList(nil);
        self:LoadList(loadedListIndex - 1);
      end
    end
    gui:exUpdateScrollFrame();
    if loadedListIndex == index then
      db.profile.cfg.loadedList = false;
      self:LoadList(nil);
      gui:clUpdateScrollFrame();
    end
  else
    local dialog = StaticPopup_Show("TORTA_CLDELETELIST", self:GetList(index).meta.listName)
    if dialog then
      dialog.data = index;
    end
  end
end

function Checklist3:RenameList(listID, newName)
  -- print(listID, newName);
  if not (type(newName) == "string") then
    local listName = self:GetList(listID).meta.listName
    local dialog = StaticPopup_Show("TORTA_CLRENAMELIST", listName);
    getglobal(dialog:GetName() .. "EditBox"):SetText(listName);
  else
    self:GetList(listID).meta.listName = newName;
    gui:exUpdateScrollFrame();
    gui:clUpdateScrollFrame();
  end
end

-- For the Trash Can, which hasn't been implemented yet
function Checklist3:RestoreList()
  
end

function Checklist3:UpdateStats()
  local checked, total, percent = 0, 0, 0;
  local list = self:GetLoadedList();
  if list then
    local checked, total = self:GetListStats();
    if total > 0 then
      percent = format("%.1f", (checked / total) * 100);
    else
      percent = "0.0";
    end
    widgets.statsButton:SetFormattedText("%d/%d (%s)", checked, total, percent .. "%");
  else
    widgets.statsButton:SetText("");
  end
end

function Checklist3:GetListStats()
  local list = self:GetLoadedList();
  if list then
    local checked = 0;
    for i = 1, #list.text do
      if list.stat[i] then
        checked = checked + 1;
      end
    end
    return checked, #list.text;
  end;
  return nil;
end

-- Drop handler functions
function Checklist3:MoveTask(SrcID, DestID)
  local list = self:GetLoadedList();
  local text = list.text[SrcID];
  local stat = list.stat[SrcID];
  
  tinsert(list.text, DestID, text);
  tinsert(list.stat, DestID, stat);
  
  if DestID < SrcID then
    SrcID = SrcID + 1;
  end
  
  tremove(list.text, SrcID);
  tremove(list.stat, SrcID);
  
  gui:clUpdateScrollFrame();
end

function Checklist3:MoveList(SrcID, DestID)
  local listsTbl = db.profile.lists;
  local list = listsTbl[SrcID];
  local loadedList = self:GetLoadedList();
  
  tinsert(listsTbl, DestID, list);
  
  if DestID < SrcID then
    SrcID = SrcID + 1;
  end
  
  tremove(listsTbl, SrcID);
  
  for i,v in ipairs(db.profile.lists) do
    if v == loadedList then
      self:LoadList();
      self:LoadList(i);
    end
  end
  
  gui:exUpdateScrollFrame();
end

-- ====================================================================
-- Search functionality
-- ====================================================================

function Checklist3:DoSearchQuery(str)
  assert(type(str) == "string", "Checklist3:DoSearchQuery(str) expects a string argument");
  wipe(searchMatches);
  wipe(searchIndices);
  widgets.exBadRegexText:Hide();
  widgets.exScrollFrame:Show();
  
  local lists = db.profile.lists;
  for index,list in ipairs(lists) do
    local listName = list.meta.listName;
    -- if ( listName:lower() ):match( str:lower() ) then
    -- if ( listName:lower() ):match( str:lower():gsub("[^$()%.[]*+-?]", "%%%1") ) then
    local valid, match = pcall(string.match, listName:lower(), str:lower());
    if not valid then
      widgets.exBadRegexText:Show();
      widgets.exScrollFrame:Hide();
    elseif valid and match then
      tinsert(searchMatches, listName);
      tinsert(searchIndices, index);
    end
  end
  
  gui:exUpdateScrollFrame();
end

function Checklist3:GetSearchMatches()
  return searchMatches, searchIndices;
end

function Checklist3:ListExists(listName)
  for i,list in ipairs(db.profile.lists) do
    if list.meta.listName == listName then
      return true;
    end
  end
  return false;
end
